/*
 * Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DPA_PING_PONG_COMMON_DEFS_H_
#define DPA_PING_PONG_COMMON_DEFS_H_

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief Number of expected receive completions on each DPA Thread
 */
#define EXPECTED_NUM_RECEIVES (100)

/**
 * @brief DPA Thread local storage struct
 */
struct dpa_ping_pong_tls {
	uint64_t received_values_arr_ptr; /**< Device memory pointer for receive values array */
	uint32_t num_receives;		  /**< Number of receive completions */
	uint32_t is_ping_thread;	  /**< Set if it's ping thread, zero if it's pong thread */
} __attribute__((__packed__, aligned(8)));

/**
 * @brief DPA Thread device argument struct
 */
struct dpa_thread_arg {
	uint64_t dpa_comp_handle;	 /**< Handle of DPA Completion Context which is attached to DPA Thread */
	uint64_t rdma_handle;		 /**< Handle of RDMA context which is attached to  DPA Completion Context */
	uint64_t recv_addr;		 /**< Receive buffer address (DPA heap memory) */
	uint64_t send_addr;		 /**< Send buffer address (DPA heap memory) */
	uint32_t recv_addr_mmap_handle;	 /**< Receive buffer DOCA Mmap handle */
	uint32_t send_addr_mmap_handle;	 /**< Send buffer DOCA Mmap handle */
	size_t recv_addr_length;	 /**< Receive buffer length */
	size_t send_addr_length;	 /**< Send buffer length */
	uint64_t comp_sync_event_handle; /**< Completion DOCA Sync Event handle */
	uint64_t comp_sync_event_val;	 /**< Completion DOCA Sync Event value to be set when application is finished */
} __dpa_global__;

#ifdef __cplusplus
}
#endif

#endif /* DPA_PING_PONG_COMMON_DEFS_H_ */
