/*
 * Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DPA_PING_PONG_CORE_H_
#define DPA_PING_PONG_CORE_H_

#include <stdlib.h>
#include <unistd.h>

#include <doca_dev.h>
#include <doca_error.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief device default name
 */
#define DEVICE_DEFAULT_NAME "NOT_SET"

/**
 * @brief A struct that includes all the resources needed for DPA
 */
struct dpa_resources {
	struct doca_dev *doca_device; /**< DOCA device for DPA */
	struct doca_dpa *doca_dpa;    /**< DOCA DPA context */
};

/**
 * @brief Configuration struct
 */
struct dpa_config {
	char device_name[DOCA_DEVINFO_IBDEV_NAME_SIZE]; /**< Buffer that holds the device name */
};

/**
 * @brief Register the command line parameters for the application
 *
 * @return: DOCA_SUCCESS on success and DOCA_ERROR otherwise
 */
doca_error_t register_dpa_params(void);

/**
 * @brief Allocate DOCA DPA resources
 *
 * @cfg [in]: DOCA DPA configurations
 * @resources [out]: DOCA DPA resources to allocate
 * @return: DOCA_SUCCESS on success and DOCA_ERROR otherwise
 */
doca_error_t allocate_dpa_resources(struct dpa_config *cfg, struct dpa_resources *resources);

/**
 * @brief Destroy DOCA DPA resources
 *
 * @resources [in]: DOCA DPA resources to destroy
 * @return: DOCA_SUCCESS on success and DOCA_ERROR otherwise
 */
doca_error_t destroy_dpa_resources(struct dpa_resources *resources);

/**
 * @brief Application Logic
 *
 * @resources [in]: DPA resources that the application will use
 * @return: DOCA_SUCCESS on success and DOCA_ERROR otherwise
 */
doca_error_t dpa_ping_pong(struct dpa_resources *resources);

#ifdef __cplusplus
}
#endif

#endif /* DPA_PING_PONG_CORE_H_ */
